@@ -1,12 +1,13 @@
 use crate::{
     allocator::Allocator,
     config::Config,
-    discref::{Page, PageOps, PAGE_METADATA_SIZE},
+    device::{Page, PageOps, PAGE_METADATA_SIZE},
     error::{GenericError, PERes, ReadError, SegmentError, TimeoutError},
     id::{PersyId, RecRef, SegmentId},
+    journal::records::{DeleteRecord, InsertRecord, NewSegmentPage, UpdateRecord},
     locks::{LockManager, RwLockManager},
     segment::{AllocatedSegmentPage, SegmentPage, SegmentPageIterator, SegmentPageRead, Segments},
-    transaction_impl::{DeleteRecord, InsertRecord, NewSegmentPage, SegmentOperation, UpdateRecord},
+    transaction_impl::SegmentOperation,
     PrepareError,
 };
 use std::{
@@ -195,12 +196,6 @@ impl Address {
         Ok(())
     }
 
-    pub fn recompute_last_pages(&self) -> PERes<()> {
-        let mut segments = self.segments.write()?;
-        segments.recompute_last_pages(&self.allocator)?;
-        Ok(())
-    }
-
     pub fn check_persistent_records(
         &self,
         records: &[(SegmentId, RecRef, u16)],
@@ -389,15 +384,14 @@ impl Address {
         Ok(())
     }
 
-    pub fn flush_segments(&self) -> PERes<()> {
-        let mut segments = self.segments.write()?;
-        segments.flush_segments(&self.allocator)
-    }
-
     pub fn exists_segment(&self, segment: &str) -> PERes<bool> {
         Ok(self.segments.read()?.has_segment(segment))
     }
 
+    pub fn exists_segment_by_id(&self, segment: &SegmentId) -> PERes<bool> {
+        Ok(self.segments.read()?.has_segment_by_id(segment))
+    }
+
     pub fn segment_id(&self, segment: &str) -> PERes<Option<SegmentId>> {
         Ok(self.segments.read()?.segment_id(segment))
     }
@@ -445,17 +439,19 @@ impl Address {
 mod tests {
     use super::Address;
     use crate::id::SegmentId;
-    use crate::transaction_impl::{CreateSegment, DeleteRecord, InsertRecord, SegmentOperation, UpdateRecord};
-    use crate::{allocator::Allocator, config::Config, discref::DiscRef};
+    use crate::journal::records::{CreateSegment, DeleteRecord, InsertRecord, UpdateRecord};
+    use crate::transaction_impl::SegmentOperation;
+    use crate::{allocator::Allocator, config::Config, device::FileDevice};
     use std::sync::Arc;
     use tempfile::Builder;
 
     fn init_test_address(file_name: &str) -> (Address, SegmentId) {
         let file = Builder::new().prefix(file_name).suffix(".persy").tempfile().unwrap();
         let config = Arc::new(Config::new());
-        let disc = Box::new(DiscRef::new(file.reopen().unwrap()).unwrap());
+        let disc = Box::new(FileDevice::new(file.reopen().unwrap()).unwrap());
         let (_, allocator) = Allocator::init(disc, &config).unwrap();
         let page = Address::init(&allocator).unwrap();
+        allocator.disc_sync().unwrap();
         let addr = Address::new(&Arc::new(allocator), &config, page).unwrap();
         let (id, fp) = addr.create_temp_segment("def").unwrap();
         addr.segments.write().unwrap().create_segment(id, fp);
