@@ -209,8 +209,11 @@ impl Allocator {
         for pos in 0..freelist.list.len() {
             let page = freelist.list[pos];
             if page != 0 {
-                let lp = dr.load_page(page)?;
-                if !lp.is_free()? {
+                if let Ok(lp) = dr.load_page(page) {
+                    if !lp.is_free()? {
+                        freelist.list[pos] = 0;
+                    }
+                } else {
                     freelist.list[pos] = 0;
                 }
             }
@@ -439,7 +442,6 @@ impl Allocator {
         holder.flush_counter = order;
         Ok(())
     }
-
     fn write_root_page_info(&self, mut info: RootWriteInfo, holder: &mut Counter) -> PERes<()> {
         let mut root = self.write_page(info.page)?;
         let last_flush = holder.flush_counter;
@@ -534,10 +536,10 @@ impl UpdateList for FreeList {
         Ok(old)
     }
 
-    fn remove(&mut self, size: u8, page: u64, next: u64) -> PERes<()> {
+    fn remove(&mut self, size: u8, page: u64, next: u64, check: bool) -> PERes<()> {
         let old = self.set(size, next);
         debug_assert!(
-            old == page || old == 0,
+            old == page || old == 0 || !check,
             "trimmed page not in top list expected:{} current:{} ",
             page,
             old
