@@ -119,6 +119,9 @@ impl<K: Clone + Ord, V: Clone> MockIndexKeeper<K, V> {
 }
 
 impl<K: Clone + Ord, V: Clone> IndexKeeper<K, V> for MockIndexKeeper<K, V> {
+    fn load_with(&self, node: &NodeRef, _reuse: Option<Nodes<K>>) -> PERes<Node<K, V>> {
+        self.load(node)
+    }
     fn load(&self, node: &NodeRef) -> PERes<Node<K, V>> {
         Ok(self.store.get(&node).unwrap().clone())
     }
@@ -339,7 +342,7 @@ fn test_add_same_value_twice() {
 }
 
 #[test]
-fn test_add_to_esclusive() {
+fn test_add_to_exclusive() {
     let mut keeper = MockIndexKeeper::<u8, u8>::new_mode(ValueMode::Exclusive);
     let mut changes = Vec::new();
     changes.push(KeyChanges::new(2, vec![ValueChange::Add(1)]));
