@@ -27,23 +27,7 @@ use std::vec::IntoIter;
 pub struct ValueIter<V> {
     value: Option<IntoIter<V>>,
 }
-/*
-impl<V:IndexType> From<Value<V>> for ValueIter<V> {
-    fn from(val: Value<V>) -> Self {
-        ValueIter {
-            value: Some(val.into_iter()),
-        }
-    }
-}
 
-impl<V:IndexType> From<Option<Value<V>>> for ValueIter<V> {
-    fn from(val: Option<Value<V>>) -> Self {
-        ValueIter {
-            value: val.map(|v| v.into_iter()),
-        }
-    }
-}
-*/
 impl<V> From<Value<V::Wrapper>> for ValueIter<V>
 where
     V: IndexTypeWrap,
