@@ -1,4 +1,3 @@
-use fs2::FileExt;
 pub use std::fs::OpenOptions;
 use std::{
     collections::HashMap,
@@ -14,7 +13,7 @@ use crate::{
     address::Address,
     allocator::Allocator,
     config::{Config, TransactionConfig},
-    discref::{Device, DiscRef, MemRef, PageOps, PAGE_METADATA_SIZE},
+    device::{Device, FileDevice, MemoryDevice, PageOps, PAGE_METADATA_SIZE},
     error::{
         CreateError, CreateIndexError, CreateSegmentError, DeleteError, DropIndexError, DropSegmentError,
         IndexChangeError, IndexError, IndexOpsError, OpenError, PERes, SegmentError, TimeoutError, UpdateError,
@@ -34,7 +33,7 @@ use crate::{
     },
     journal::{Journal, JournalId},
     record_scanner::{SegmentRawIter, SegmentSnapshotRawIter, TxSegmentRawIter},
-    snapshots::{release_snapshot, EntryCase, SegmentSnapshop, SnapshotId, Snapshots},
+    snapshots::{release_snapshot, EntryCase, SegmentSnapshot, SnapshotId, Snapshots},
     transaction_impl::{PreparedState, SyncMode, TransactionImpl, TxRead, TxSegCheck},
     PrepareError, ReadError, RecoverStatus,
 };
@@ -151,11 +150,24 @@ fn create_background_ops(
             for snap in all {
                 release_snapshot(*snap, &snapshots, &allocator, &journal)?;
             }
+            if !all.is_empty() {
+                journal.clear_in_queue()?;
+            }
             Ok(())
         },
     )
 }
 
+pub trait RecordReader<T> {
+    fn read(self, rec: ArcSliceRead) -> T;
+}
+
+impl<T, F: FnOnce(ArcSliceRead) -> T> RecordReader<T> for F {
+    fn read(self, rec: ArcSliceRead) -> T {
+        (self)(rec)
+    }
+}
+
 impl PersyImpl {
     pub fn create(path: &Path) -> Result<(), CreateError> {
         let f = OpenOptions::new().write(true).read(true).create_new(true).open(path)?;
@@ -163,13 +175,7 @@ impl PersyImpl {
     }
 
     pub fn create_from_file(f: File) -> Result<(), CreateError> {
-        f.try_lock_exclusive()?;
-        PersyImpl::init_file(f)?;
-        Ok(())
-    }
-
-    fn init_file(fl: File) -> PERes<()> {
-        PersyImpl::init(Box::new(DiscRef::new(fl)?))?;
+        PersyImpl::init(Box::new(FileDevice::new(f)?))?;
         Ok(())
     }
 
@@ -303,23 +309,36 @@ impl PersyImpl {
         for (_, (_, tx, _)) in recover.transactions.iter_mut() {
             tx.recover_rollback(self)?;
         }
-        self.address.recompute_last_pages()?;
-        self.address.flush_segments()?;
         allocator.disc_sync()?;
-        self.journal.clear_in_queque()?;
+        self.journal.clear_in_queue()?;
         allocator.trim_free_at_end()?;
         Ok(())
     }
 
+    pub fn truncate_and_open(f: File, config: Config) -> Result<(PersyImpl, RecoverImpl), OpenError> {
+        let device = Box::new(FileDevice::new_truncate(f)?);
+        let device = PersyImpl::init(device)?;
+        let persy = PersyImpl::new(device, config)?;
+        let rec = persy.recover()?;
+        Ok((persy, rec))
+    }
+
+    pub fn create_and_open(f: File, config: Config) -> Result<(PersyImpl, RecoverImpl), OpenError> {
+        let device = Box::new(FileDevice::new(f)?);
+        let device = PersyImpl::init(device)?;
+        let persy = PersyImpl::new(device, config)?;
+        let rec = persy.recover()?;
+        Ok((persy, rec))
+    }
+
     pub fn open_recover(f: File, config: Config) -> Result<(PersyImpl, RecoverImpl), OpenError> {
-        f.try_lock_exclusive()?;
-        let persy = PersyImpl::new(Box::new(DiscRef::new(f)?), config)?;
+        let persy = PersyImpl::new(Box::new(FileDevice::new(f)?), config)?;
         let rec = persy.recover()?;
         Ok((persy, rec))
     }
 
     pub fn memory(config: Config) -> Result<PersyImpl, OpenError> {
-        let device = PersyImpl::init(Box::new(MemRef::new()?))?;
+        let device = PersyImpl::init(Box::new(MemoryDevice::new()?))?;
         PersyImpl::new(device, config)
     }
 
@@ -371,6 +390,10 @@ impl PersyImpl {
         self.address.exists_segment(segment)
     }
 
+    pub fn exists_segment_by_id(&self, segment: &SegmentId) -> PERes<bool> {
+        self.address.exists_segment_by_id(segment)
+    }
+
     pub fn exists_segment_tx(&self, tx: &TransactionImpl, segment: &str) -> PERes<bool> {
         match tx.exists_segment(segment) {
             TxSegCheck::Dropped => Ok(false),
@@ -472,7 +495,7 @@ impl PersyImpl {
         let page = pg.get_index();
         tx.add_insert(&self.journal, segment_id, &rec_ref, page)?;
         if let Some(new_page) = maybe_new_page {
-            tx.add_new_segment_page(&self.journal, segment_id, new_page.new_page, new_page.previus_page)?;
+            tx.add_new_segment_page(&self.journal, segment_id, new_page.new_page, new_page.previous_page)?;
         }
         pg.write_all(&metadata);
         pg.write_all(rec);
@@ -490,7 +513,7 @@ impl PersyImpl {
             .address
             .snapshot_list()?
             .into_iter()
-            .map(|(name, id, first_page)| SegmentSnapshop::new(&name, id, first_page))
+            .map(|(name, id, first_page)| SegmentSnapshot::new(&name, id, first_page))
             .collect::<Vec<_>>();
         self.snapshots.fill_segments(snapshot_id, &segs)?;
         Ok(snapshot_id)
@@ -517,14 +540,14 @@ impl PersyImpl {
     }
 
     fn read_page(&self, match_id: &RecRef, page: u64) -> PERes<Option<Vec<u8>>> {
-        self.read_page_fn(match_id, page, |x| x.to_vec())
+        self.read_page_fn(match_id, page, |x: ArcSliceRead| x.to_vec())
     }
 
-    fn read_page_fn<T, F: Fn(ArcSliceRead) -> T>(&self, match_id: &RecRef, page: u64, f: F) -> PERes<Option<T>> {
+    fn read_page_fn<T, F: RecordReader<T>>(&self, match_id: &RecRef, page: u64, f: F) -> PERes<Option<T>> {
         if let Some(mut pg) = self.allocator.load_page_not_free(page)? {
             let (len, id) = PersyImpl::read_record_metadata(&mut pg);
             if id.page == match_id.page && id.pos == match_id.pos {
-                Ok(Some(f(pg.arc_slice(len as usize))))
+                Ok(Some(f.read(pg.arc_slice(len as usize))))
             } else {
                 Ok(None)
             }
@@ -622,15 +645,15 @@ impl PersyImpl {
         rec_ref: &RecRef,
         snapshot: SnapshotId,
     ) -> Result<Option<Vec<u8>>, ReadError> {
-        self.read_snap_fn(segment, rec_ref, snapshot, |x| x.to_vec())
+        self.read_snap_fn(segment, rec_ref, snapshot, |x: ArcSliceRead| x.to_vec())
     }
 
-    pub fn read_snap_fn<T>(
+    pub fn read_snap_fn<T, F: RecordReader<T>>(
         &self,
         segment: SegmentId,
         rec_ref: &RecRef,
         snapshot: SnapshotId,
-        f: fn(ArcSliceRead) -> T,
+        f: F,
     ) -> Result<Option<T>, ReadError> {
         let segment_id = segment;
         if let Some(rec_vers) = self.snapshots.read(snapshot, rec_ref)? {
@@ -1043,13 +1066,15 @@ impl PersyImpl {
         match sync_mode {
             SyncMode::Sync => {
                 let allocator = self.allocator();
-                allocator.disc_sync()
+                allocator.disc_sync()?;
+                self.journal().clear_in_queue()?;
             }
             SyncMode::BackgroundSync => {
                 self.snapshots.acquire(snapshot_id)?;
-                self.background_ops.add_pending(snapshot_id)
+                self.background_ops.add_pending(snapshot_id)?;
             }
         }
+        Ok(())
     }
 
     #[cfg(not(feature = "background_ops"))]
@@ -1057,14 +1082,18 @@ impl PersyImpl {
         match sync_mode {
             SyncMode::Sync => {
                 let allocator = self.allocator();
-                allocator.disc().sync()
+                allocator.disc().sync()?;
+                self.journal().clear_in_queue()?;
             }
             SyncMode::BackgroundSync => unreachable!(),
         }
+        Ok(())
     }
 }
 impl Drop for PersyImpl {
     fn drop(&mut self) {
+        #[cfg(feature = "background_ops")]
+        self.background_ops.finish();
         self.allocator.flush_free_list().unwrap();
         self.allocator.disc_sync().unwrap();
     }
