@@ -4,9 +4,8 @@ use crate::{
         FLAG_EXISTS, SEGMENT_DATA_OFFSET, SEGMENT_HASH_OFFSET, SEGMENT_PAGE_DELETE_COUNT_OFFSET,
     },
     allocator::Allocator,
-    discref::{Page, PageOps},
+    device::{Page, PageOps},
     error::PERes,
-    flush_checksum::double_buffer_check,
     id::{RecRef, SegmentId},
     io::{read_u64, write_u64, InfallibleRead, InfallibleReadFormat, InfallibleWrite, InfallibleWriteFormat},
     persy::exp_from_content_size,
@@ -24,7 +23,7 @@ const SEGMENTS_ROOT_PAGE_VERSION: u8 = SEGMENTS_ROOT_PAGE_VERSION_0;
 
 pub struct AllocatedSegmentPage {
     pub new_page: u64,
-    pub previus_page: u64,
+    pub previous_page: u64,
 }
 
 pub struct Segment {
@@ -109,7 +108,7 @@ impl Segment {
                     RecRef::new(self.last_page, SEGMENT_DATA_OFFSET),
                     Some(AllocatedSegmentPage {
                         new_page,
-                        previus_page: page,
+                        previous_page: page,
                     }),
                 )
             }
@@ -332,7 +331,6 @@ pub struct Segments {
     pub root_page: u64,
     content_page: u64,
     other_page: u64,
-    last_flush: u8,
     pub segments: HashMap<SegmentId, Segment>,
     pub segments_id: HashMap<String, SegmentId>,
     pub temp_segments: HashMap<SegmentId, Segment>,
@@ -351,26 +349,15 @@ pub fn segment_hash(segment: &str) -> u64 {
 
 impl Segments {
     pub fn new(root_page: u64, allocator: &Arc<Allocator>) -> PERes<Segments> {
-        let mut buffer_0 = [0; 19];
-        let mut buffer_1 = [0; 19];
         let page_id;
         let other_page_id;
-        let last_flush;
         {
             let mut root = allocator.load_page(root_page)?;
             match root.read_u8() {
                 SEGMENTS_ROOT_PAGE_VERSION_0 => {
-                    root.read_exact(&mut buffer_0);
-                    root.read_exact(&mut buffer_1);
-                    let (flush, first) = double_buffer_check(&buffer_0, &buffer_1);
-                    last_flush = flush;
-                    if first {
-                        page_id = read_u64(&buffer_0[0..8]);
-                        other_page_id = read_u64(&buffer_0[8..16]);
-                    } else {
-                        page_id = read_u64(&buffer_1[0..8]);
-                        other_page_id = read_u64(&buffer_1[8..16]);
-                    }
+                    let buffer = allocator.read_root_address(&mut root, 19);
+                    page_id = read_u64(&buffer[0..8]);
+                    other_page_id = read_u64(&buffer[8..16]);
                 }
                 _ => panic!("version not supported"),
             }
@@ -397,7 +384,6 @@ impl Segments {
             root_page,
             content_page: page_id,
             other_page: other_page_id,
-            last_flush,
             segments,
             segments_id,
             temp_segments: HashMap::new(),
@@ -409,7 +395,7 @@ impl Segments {
         let mut buffer = [0; 19];
         write_u64(&mut buffer[0..8], 0);
         write_u64(&mut buffer[8..16], 0);
-        allocator.write_address_root(root, &mut buffer, SEGMENTS_ROOT_PAGE_VERSION, 0)?;
+        allocator.write_address_root(root, &mut buffer, SEGMENTS_ROOT_PAGE_VERSION)?;
         Ok(())
     }
 
@@ -436,6 +422,9 @@ impl Segments {
     pub fn has_segment(&self, segment: &str) -> bool {
         self.segments_id.contains_key(segment)
     }
+    pub fn has_segment_by_id(&self, segment: &SegmentId) -> bool {
+        self.segments.contains_key(segment)
+    }
 
     pub fn create_temp_segment(&mut self, allocator: &Allocator, segment: &str) -> PERes<(SegmentId, u64)> {
         let mut allocated = allocator.allocate(ADDRESS_PAGE_EXP)?;
@@ -522,20 +511,6 @@ impl Segments {
         Ok(())
     }
 
-    pub fn recompute_last_pages(&mut self, allocator: &Allocator) -> PERes<()> {
-        for (_, segment) in self.segments.iter_mut() {
-            loop {
-                let mut page = allocator.load_page(segment.last_page)?;
-                let n_page = page.get_next()?;
-                if n_page == 0 {
-                    break;
-                }
-                segment.last_page = n_page;
-            }
-        }
-        Ok(())
-    }
-
     pub fn flush_segments(&mut self, allocator: &Allocator) -> PERes<()> {
         let mut buffer = Vec::<u8>::new();
         for segment in self.segments.values() {
@@ -566,8 +541,7 @@ impl Segments {
         write_u64(&mut root_buffer[0..8], content_page_id);
         write_u64(&mut root_buffer[8..16], other_page_id);
         let root = allocator.write_page(self.root_page)?;
-        self.last_flush =
-            allocator.write_address_root(root, &mut root_buffer, SEGMENTS_ROOT_PAGE_VERSION, self.last_flush)?;
+        allocator.write_address_root(root, &mut root_buffer, SEGMENTS_ROOT_PAGE_VERSION)?;
         self.content_page = content_page_id;
         self.other_page = other_page_id;
         Ok(())
@@ -644,7 +618,7 @@ mod tests {
         address::ADDRESS_PAGE_EXP,
         allocator::Allocator,
         config::Config,
-        discref::{DiscRef, Page, PageOps},
+        device::{FileDevice, Page, PageOps},
         id::SegmentId,
     };
     use std::sync::Arc;
@@ -659,7 +633,7 @@ mod tests {
             .reopen()
             .unwrap();
         let config = Arc::new(Config::new());
-        let disc = Box::new(DiscRef::new(file).unwrap());
+        let disc = Box::new(FileDevice::new(file).unwrap());
         let (_, allocator) = Allocator::init(disc, &config).unwrap();
         allocator.allocate(5).unwrap(); //Just to be sure it not start from 0, it cannot happen in not test cases.
 
@@ -673,6 +647,7 @@ mod tests {
         let root = all.allocate(ADDRESS_PAGE_EXP).unwrap();
         let root_index = root.get_index();
         Segments::init(root, &all).unwrap();
+        all.disc_sync().unwrap();
         let mut segments = Segments::new(root_index, &all).unwrap();
 
         let (id, fp) = segments.create_temp_segment(&all, "some").unwrap();
@@ -693,6 +668,7 @@ mod tests {
         let root = all.allocate(ADDRESS_PAGE_EXP).unwrap();
         let root_index = root.get_index();
         Segments::init(root, &all).unwrap();
+        all.disc_sync().unwrap();
         let mut segments = Segments::new(root_index, &all).unwrap();
 
         let (id, _) = segments.create_temp_segment(&all, "some").unwrap();
@@ -710,6 +686,7 @@ mod tests {
         let root = all.allocate(ADDRESS_PAGE_EXP).unwrap();
         let root_index = root.get_index();
         Segments::init(root, &all).unwrap();
+        all.disc_sync().unwrap();
         let id;
         {
             let mut segments = Segments::new(root_index, &all).unwrap();
@@ -717,6 +694,7 @@ mod tests {
             id = c_id;
             segments.create_segment(id, fp);
             segments.flush_segments(&all).unwrap();
+            all.disc_sync().unwrap();
         }
         {
             let mut segments = Segments::new(root_index, &all).unwrap();
@@ -725,6 +703,7 @@ mod tests {
             assert!(segments.segments.contains_key(&id));
             segments.drop_segment("some");
             segments.flush_segments(&all).unwrap();
+            all.disc_sync().unwrap();
         }
         {
             let segments = Segments::new(root_index, &all).unwrap();
@@ -740,6 +719,7 @@ mod tests {
         let root = all.allocate(ADDRESS_PAGE_EXP).unwrap();
         let root_index = root.get_index();
         Segments::init(root, &all).unwrap();
+        all.disc_sync().unwrap();
         {
             let mut segments = Segments::new(root_index, &all).unwrap();
             for i in 0..100 {
@@ -747,6 +727,7 @@ mod tests {
                 segments.create_segment(id, fp);
             }
             segments.flush_segments(&all).unwrap();
+            all.disc_sync().unwrap();
         }
         {
             let mut segments = Segments::new(root_index, &all).unwrap();
@@ -755,6 +736,7 @@ mod tests {
                 segments.drop_segment(&format!("some{}", i));
             }
             segments.flush_segments(&all).unwrap();
+            all.disc_sync().unwrap();
         }
         {
             let segments = Segments::new(root_index, &all).unwrap();
