@@ -722,7 +722,7 @@ mod tests {
         let val;
         {
             let persy = Persy::open("./target/test_recover_prepared.persy", Config::new()).unwrap();
-            let mut tx = persy.begin().expect("error on transactoin begin");
+            let mut tx = persy.begin().expect("error on transaction begin");
             tx.create_segment("def").expect("error on segment creation");
             let fin = tx.prepare().expect("error on commit prepare");
             fin.commit().expect("error on commit");
@@ -742,7 +742,7 @@ mod tests {
     }
 
     #[test]
-    pub fn test_dobule_recover_prepared_tx() {
+    pub fn test_double_recover_prepared_tx() {
         Persy::create("./target/test_double_recover_prepared.persy").unwrap();
         let id;
         let id1;
@@ -750,7 +750,7 @@ mod tests {
         let val1;
         {
             let persy = Persy::open("./target/test_double_recover_prepared.persy", Config::new()).unwrap();
-            let mut tx = persy.begin().expect("error on transactoin begin");
+            let mut tx = persy.begin().expect("error on transaction begin");
             tx.create_segment("def").expect("error on segment creation");
             let fin = tx.prepare().expect("error on commit prepare");
             fin.commit().expect("error on commit");
@@ -791,7 +791,7 @@ mod tests {
         {
             val = String::from("aaa").into_bytes();
             let persy = Persy::open("./target/test_recover_tx_id.persy", Config::new()).unwrap();
-            let mut tx = persy.begin().expect("error on transactoin begin");
+            let mut tx = persy.begin().expect("error on transaction begin");
             tx.create_segment("def").expect("error on segment creation");
             id_pers = tx.insert("def", &val).expect("error on insert value");
             id_pers_update = tx.insert("def", &val).expect("error on insert value");
@@ -839,7 +839,7 @@ mod tests {
         {
             val = String::from("aaa").into_bytes();
             let persy = Persy::open("./target/test_recover_tx_choice.persy", Config::new()).unwrap();
-            let mut tx = persy.begin().expect("error on transactoin begin");
+            let mut tx = persy.begin().expect("error on transaction begin");
             tx.create_segment("def").expect("error on segment creation");
             id_pers = tx.insert("def", &val).expect("error on insert value");
             id_pers_update = tx.insert("def", &val).expect("error on insert value");
